# -*- encoding: utf-8 -*-
from openerp.osv import fields, osv
from openerp.tools.translate import _
from openerp.tools.misc import DEFAULT_SERVER_DATE_FORMAT
from .util import get_secure_filename
import sys
import re

if sys.platform == 'win32':
    import pbs as sh
else:
    import sh

import os
from base64 import b64encode
import datetime

import logging
_logger = logging.getLogger(__name__)


class job_definition(osv.Model):
    _name = 'external_job.job_definition'

    _columns = {
        'name': fields.char(
            u"Name",
            size=128,
            required=True,
        ),
        'command_line': fields.char(
            u"Command Line",
            size=512,
            help=u"The program to call",
            required=True,
        ),
        'arguments': fields.char(
            u"Arguments",
            size=256,
            help=(
                u"Optional arguments that will be placed placed after the "
                u"database name and the job ID."
            ),
        ),
        'config_arguments': fields.char(
            u"Arguments from Configuration",
            size=256,
            help=(
                u"Arguments to extract from the current configuration, placed "
                u"after the static arguments.\n"
                u"Please note that the db_name configuration key does NOT "
                u"correspond to the active database, which is determined from "
                u"the cursor and always passed to the job as first argument."
            )
        ),
        'short_desc': fields.char(
            u"Short Description",
            size=256,
        ),
        'long_desc': fields.text(
            u"Long Description",
        ),
        'file_extension': fields.char(
            u"File extension",
            size=12,
            help=u"The desired file extension excluding the dot. Ex: csv",
            required=True,
        ),
        'model_id': fields.many2one(
            'ir.model',
            string=u"Model",
            ondelete="cascade",
            help=u"The data model our job is expecting to process",
            required=True,
        ),
    }

    def _check_file_extension(self, cr, uid, ids, context=None):
        records = self.read(cr, uid, ids, ['file_extension'], context=context)
        for record in records:
            file_extension = record['file_extension']
            if re.search(r"\s", file_extension, flags=re.UNICODE):
                return False
        return True

    _constraints = [
        (
            _check_file_extension,
            u"Avoid spaces in file extensions",
            ['file_extension']
        ),
    ]

    def run_job(self, cr, uid, ids, context=None):
        """This callback method is called by a user pressing a button
        in the form view. When the user presses the button we run the command
        and collect the data back for him.
        WARNING: this is not intended to use by API. If you want to use API,
        look at the run_jobs method
        """
        # since we want to return a window to download the data we need to
        # only execute one job...
        # if you want to execute multiple jobs use the run_jobs method instead

        id_ = ids[0]
        job = self.browse(cr, uid, id_, context=context)

        if not job.command_line:
            _logger.warning(_(u"No command line for given job"))
            return False
        else:
            command = job.command_line

        suffix = '.%s' % job.file_extension if job.file_extension else ""
        joblog_osv = self.pool.get('external_job.job_log')
        out_file_name = get_secure_filename(suffix=suffix)
        in_file_name = get_secure_filename()

        if context.get('resource_ids', None):
            # if the caller has injected specific resources ids in the context
            # we will dump them one by one in the input file and give this
            # to the joblog definition so that the job runner (ie: the external
            # script) can get to them and do whatever it needs with it.
            # This mechanism is usefull for specialized commands when we want
            # to create context menus in some list views and extract only the
            # selected ids by passing them in the context
            resource_ids = context.get('resource_ids')
            with open(in_file_name, 'w+') as f:
                for resource_id in resource_ids:
                    f.write("%s\n" % resource_id)

        create_args = {
            'job_definition_id': id_,
            'in_file_name': in_file_name,
            'out_file_name': out_file_name,
        }

        joblog_id = joblog_osv.create(cr, uid, create_args, context=context)

        # Make sure we commit our transaction before giving the newly created
        # id to our external process or it won't be able to find it :)
        cr.commit()

        args = [cr.dbname, str(joblog_id)]
        if job.arguments:
            for argument in job.arguments.split():
                args.append(argument)

        # Append wizard arguments put in context
        for argument in context["args"]:
            for k, v in argument.iteritems():
                args.append(k)
                args.append(v)

        try:
            run = sh.Command(command)
        except sh.CommandNotFound:
            _logger.info('path is %s', os.environ['PATH'])
            raise osv.except_osv(
                _("Error"),
                _("Command %s not found") % command)

        if args:
            out = run(*args)
        else:
            out = run()

        with open(out_file_name, 'rb') as outfile:
            outfile_data = outfile.read()

            if not outfile_data:
                outfile_data = ''

        outfile_data = b64encode(outfile_data)

        write_args = {
            'end_date': fields.datetime.now(),
            'stdout': out,
        }
        joblog_osv.write(cr, uid, joblog_id, write_args, context=context)

        attachment_osv = self.pool.get('ir.attachment')

        fname = "%s-%s.%s" % (
            job.name,
            datetime.datetime.now().date().strftime(
                DEFAULT_SERVER_DATE_FORMAT),
            job.file_extension,
        )

        # cleanup temp
        os.remove(in_file_name)
        os.remove(out_file_name)

        create_args = {
            'name': fname,
            'datas': outfile_data,
            'datas_fname': fname,
            'res_id': joblog_id,
            'res_model': 'external_job.job_log',
        }
        attachment_osv.create(cr, uid, create_args, context=context)

        return {
            'name': 'Job Log',
            'type': 'ir.actions.act_window',
            'res_model': 'external_job.job_log',
            'res_id': joblog_id,
            'view_type': 'form',
            'view_mode': 'form',
            'context': context,
            'target': 'new',
        }
